/*      $Id: irmix.c,v 5.3 2000/07/08 08:17:48 columbus Exp $      */

/****************************************************************************
 ** irmix.c ****************************************************************
 ****************************************************************************
 *
 * irmix  - adjust volume of audio channels using lirc commands
 *
 *
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <errno.h>
#include <unistd.h>
#include <signal.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <lirc/lirc_client.h>

#include "tokenize.h"
#include "mix.h"
#include "osd.h"

char *progname=PACKAGE "-" VERSION;

enum myids {id_null,id_set,id_inc,id_dec,id_mute,id_unmute,id_togglemute,id_nextchan,id_prevchan,id_select,id_record};

struct easyoption myoptions[]=
{
  {id_set,"set",2,0,0,0,0},
  {id_inc,"inc",2,0,0,0,0},
  {id_dec,"dec",2,0,0,0,0},
  {id_mute,"mute",1,0,0,0,0},
  {id_unmute,"unmute",1,0,0,0,0},
  {id_togglemute,"togglemute",1,0,0,0,0},
  {id_nextchan,"nextchannel",0,0,0,0,0},
  {id_prevchan,"prevchannel",0,0,0,0,0},
    {id_select, "select",1,0,0,0,0},
  {id_record,"record",1,0,0,0,0},
  {0,NULL,0,0,0,0,0}
};

enum optresults {opt_null,opt_daemonize,opt_exit_failure,opt_exit_success};

/* Globals */

static long osd_utime;
static struct osdcontext *irmix_osd;
static int alarm_pending;

/* set the mixer  */
#ifndef DEFAULT_MIXERDEVICE
#ifdef HAVE_LIBASOUND
#define DEFAULT_MIXERDEVICE "hw:0";
#else
#define DEFAULT_MIXERDEVICE "/dev/mixer";
#endif
#endif

char device[255] = DEFAULT_MIXERDEVICE;
int screen=0;

void sigalarm_handler(int junk)
{
  if(alarm_pending)
  {
    irmix_osd->hide(irmix_osd);
    alarm_pending=0;
  }
}


int process_options(int argc,char *argv[])
{
  int daemonize=0;
  while(1)
  {
    int c;
    static struct option long_options[] =
    {
      {"help",no_argument,NULL,'h'},
      {"version",no_argument,NULL,'v'},
      {"daemon",no_argument,NULL,'d'},
      {"mixer",required_argument,NULL,'m'},
      {"screen",required_argument,NULL,'s'},
      {0, 0, 0, 0}
    };
    c = getopt_long(argc,argv,"hvdms:",long_options,NULL);
    if(c==-1)
      break;
    switch (c)
    {
    case 'h':
      printf("Usage: %s [options]\n",argv[0]);
      printf("\t -h --help\t\tdisplay this message\n");
      printf("\t -v --version\t\tdisplay version\n");
      printf("\t -d --daemon\t\trun in background\n");
      printf("\t -m --mixer=/dev/mixer<n>\t\tuse alternate mixer device\n");
#ifdef HAVE_LIBXINERAMA
      printf("\t -s --screen=0\t\tuse a Xinerama screen other than screen 0\n");
#endif /* HAVE_LIBXINERAMA */
      return(opt_exit_success);
    case 'v':
      printf("%s\n",progname);
      return(opt_exit_success);
    case 'd':
      daemonize=1;
      break;
    case 'm':
      if(optarg)
        strcpy(device,optarg);
      break;
#ifdef HAVE_LIBXINERAMA
    case 's':
      if(optarg)
        screen = atoi(optarg);
      break;
#endif /* HAVE_LIBXINERAMA */
    default:
      printf("Usage: %s [options]\n",argv[0]);
      return(opt_exit_failure);
    }
  }
  if (optind < argc-1)
  {
    fprintf(stderr,"%s: too many arguments\n",progname);
    return(opt_exit_failure);
  }
  return((daemonize ? opt_daemonize : opt_null));
}


void doosd(struct osdcontext *osd,struct mixer *mix,int channel)
{
  static int lastchannel=-1;
  int level;
  if(osd)
  {
    alarm(0);
    if(channel!=lastchannel)
    {
      lastchannel=channel;
      osd->setname(osd,mix->getchannelname(mix,channel),
                   mix->getchanneldev(mix,channel));
    }
    mix->getvolume(mix,channel,&level,&level);
    osd->show(osd);
    osd->setlevel(osd,level,mix->getmute(mix, channel));

    alarm_pending=1;
    alarm(2);
  }
}


int main(int argc, char *argv[])
{
  struct lirc_config *config;
  int daemonize=0;
  int currentchannel=0;
  struct mixer *mix;
  struct osdcontext *osd;

  switch(process_options(argc,argv))
  {
    case opt_exit_success:
      return(EXIT_SUCCESS); break;
    case opt_exit_failure:
      return(EXIT_FAILURE); break;
    case opt_daemonize:
      daemonize=1; break;
  }

  if(!(mix=mixer_create(device)))
    return(EXIT_FAILURE);

  osd=irmix_osd=osd_create(100,30,screen);

  if(osd)
    osd->setmaxlevel(osd,MAX_MIXER_VALUE);

  signal(SIGALRM,sigalarm_handler);

  if(lirc_init("irmix",daemonize ? 0:1)==-1) exit(EXIT_FAILURE);

  if(lirc_readconfig(optind!=argc ? argv[optind]:NULL,&config,NULL)==0)
  {
    char *code;
    char *c;
    int ret;

    if(daemonize)
    {
      if(daemon(0,0)==-1)
      {
        fprintf(stderr,"%s: can't daemonize\n",
          progname);
        perror(progname);
        lirc_freeconfig(config);
        lirc_deinit();
        exit(EXIT_FAILURE);
      }
    }
    while(lirc_nextcode(&code)==0)
    {
      if(code==NULL) continue;
      while((ret=lirc_code2char(config,code,&c))==0 && c!=NULL)
      {
        struct easyoption *opt=parseoption(myoptions,c);
        if (opt)
        {
          switch(opt->id)
          {
            case id_set:
              if(opt->argsread>1)
                currentchannel=opt->value2;
              mix->setvolume(mix,currentchannel,opt->value1,opt->value1);
              doosd(osd,mix,currentchannel);
              break;
            case id_inc:
              if(opt->argsread>1)
                currentchannel=opt->value2;
              mix->incvolume(mix,currentchannel,opt->value1,opt->value1);
              doosd(osd,mix,currentchannel);
              break;
            case id_dec:
              if(opt->argsread>1)
                currentchannel=opt->value2;
              mix->decvolume(mix,currentchannel,opt->value1,opt->value1);
              doosd(osd,mix,currentchannel);
              break;
            case id_nextchan:
              ++currentchannel; currentchannel%=mix->devs;
              doosd(osd,mix,currentchannel);
              break;
            case id_prevchan:
              --currentchannel;
              if(currentchannel<0)
                currentchannel+=mix->devs;
              doosd(osd,mix,currentchannel);
              break;
            case id_select:
              if(opt->argsread>0)
                currentchannel=opt->value1;
              doosd(osd,mix,currentchannel);
              break;
            case id_mute:
              if(opt->argsread>0)
                currentchannel=opt->value1;
              mix->mute(mix,currentchannel);
              doosd(osd,mix,currentchannel);
              break;
            case id_unmute:
              if(opt->argsread>0)
                currentchannel=opt->value1;
              mix->unmute(mix,currentchannel);
              doosd(osd,mix,currentchannel);
              break;
            case id_togglemute:
              if(opt->argsread>0)
                currentchannel=opt->value1;
              mix->togglemute(mix,currentchannel);
              doosd(osd,mix,currentchannel);
              break;
            case id_record:
              if(opt->argsread>0)
                currentchannel=opt->value1;
              mix->record(mix,currentchannel);
              doosd(osd,mix,currentchannel);
              break;
          }
        }
        else
          fprintf(stderr,"Command not recognised: %s\n",c);
      }
      free(code);
      if(ret==-1) break;
    }
    lirc_freeconfig(config);
  }

  lirc_deinit();
  mix->dispose(mix); mix=NULL;

  exit(EXIT_SUCCESS);
}
